package edu.kit.iti.depgraph;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class ReferencedClasses {
	
	private static Map<String, Set<String>> referencedClasses = new TreeMap<String, Set<String>>();;

	public static void main(String[] args) throws IOException {
		
		if(args[0].startsWith("@")) {
			BufferedReader br = new BufferedReader(new FileReader(args[0].substring(1)));
			String line = br.readLine();
			while(line != null) {
				check(line);
				line = br.readLine();
			}
		} else {
			for (String arg : args) {
				check(arg);
			}
		}
		
		if(Boolean.getBoolean("FULL")) {
			fullReport();
		} else {
			perClassReport();
		}
		
	}

	private static void perClassReport() {
		for (Map.Entry<String, Set<String>> entry : referencedClasses.entrySet()) {
			System.out.println(entry.getKey() + ":");
			for (String from : entry.getValue()) {
				System.out.println("  " + from);
			}
		}
	}
	
	private static void fullReport() {
		for (Map.Entry<String, Set<String>> entry : referencedClasses.entrySet()) {
			for (String from : entry.getValue()) {
				System.out.println(entry.getKey() + ":" + from);
			}
		}
	}

	private static void check(String arg) throws IOException {
		ClassPool cp = new ClassPool(arg);
		
		String myClassName = cp.getClassName(cp.getThisClassIndex()).replace('/', '.');
		
		for (int i = 0; i < cp.getCount(); i++) {
			int type = cp.getConstantType(i);
			if(type == ClassPool.CONSTANT_Class) {
				String constant = cp.getClassName(i);
				addReferencedClass(constant, myClassName);
			}
		}		
	}

	private static void addReferencedClass(String className, String from) {
		
		if(className.startsWith("[")) {
			int start = 1;
			while(className.charAt(start) == '[')
				start++;
			if(className.charAt(start) != 'L')
				return;
			className = className.substring(start+1, className.length() - 1);
		}
		
		className = className.replace('/', '.');
		
		Set<String> set = referencedClasses.get(className);
		if(set==null) {
			set = new TreeSet<String>();
			referencedClasses.put(className, set);
		}
		
		set.add(from);
	}

	
}
